/**
  Draw multiple objects in a single raytracer pass.
  
  Dr. Orion Sky Lawlor, olawlor@acm.org, 2012-01-30 (Public Domain)
*/
#include "physics/world.h" /* physics::library and physics::object */
#include "ogl/glsl.h"

/* Just include library bodies here, for easy linking */
#include "physics/world.cpp" 
#include "physics/config.cpp"
#include "ogl/glsl.cpp"


/* A raytraced object */
class rayObject : public physics::object {
public:
	rayObject(void) 
		:physics::object(0.01)  /* <- our timestep, in seconds */
	{ }
	
	void simulate(physics::library &lib) { }
	
	void draw(physics::library &lib) {
		/* Load up shader */
		GLhandleARB prog=makeProgramObjectFromFiles(
			"vertex.txt","fragment.txt"
		);
		
		glUseProgramObjectARB(prog);
		unsigned int ul=glGetUniformLocationARB(prog,"camera");
		glUniform3fvARB(ul,1,camera);

		/* Draw raytracer proxy geometry *HUGE* (to cover everything) */
		glColor4f(1.0f,1.0f,1.0f,0.4f);
		glutSolidSphere(500.0,4,8);
		
		glUseProgramObjectARB(0);
		
		/* Draw non-raytraced geometry */
		glColor4f(0.0f,1.0f,0.0f,0.4f);
		glutSolidSphere(0.7,40,80);
	}
};


/* Called create a new simulation */
void physics_setup(physics::library &lib) {
	lib.world->add(new rayObject());
	lib.background[0]=lib.background[1]=0.6;
	lib.background[2]=1.0; // light blue
}

/* Called every frame to draw everything */
void physics_draw_frame(physics::library &lib) {
	/* Point and line smoothing needs usual alpha blending */
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	
	/* Draw opaque white ground plane */
	float groundz=-1.0;
	float groundsize=30;
	glEnable(GL_DEPTH_TEST);
	glColor3f(1,1,1);
	glBegin(GL_QUADS);
	glVertex3f(-groundsize,-groundsize,groundz);
	glVertex3f(+groundsize,-groundsize,groundz);
	glVertex3f(+groundsize,+groundsize,groundz);
	glVertex3f(-groundsize,+groundsize,groundz);
	glEnd();
	
	/* No depth while drawing ground grid */
	glDisable(GL_DEPTH_TEST);
	glColor4f(0.0,0.0,0.0,1.0);
	glEnable(GL_LINE_SMOOTH);
	groundz-=0.001;
	glLineWidth(2.0);
	if (camera.z>=groundz) { /* no grid if you're underneath ground */
		glBegin(GL_LINES);
		for (float grid=-groundsize;grid<=+groundsize;grid++) {
			glVertex3f(-groundsize,grid,groundz); 
			glVertex3f(+groundsize,grid,groundz); 
			glVertex3f(grid,-groundsize,groundz); 
			glVertex3f(grid,+groundsize,groundz); 
		}
		glEnd();
	}
	
	/* Do depth while drawing objects */
	glEnable(GL_DEPTH_TEST);
	lib.world->draw(lib);
	
	
}


